-----------------------------------------------------------------------------------------------
--
--
--
-----------------------------------------------------------------------------------------------

local addon = KiwiPlates

--================================================

local selectedSkin
local selectedSkinIndex
local skinCommand
local newSkinName

--================================================

addon.OptionsTable = { name = "KiwiPlates Options", type = "group", childGroups = "tab", args = {
	General = { type = "group", order = 10, name = 'General', childGroups = nil, args = {} },
	Skins = { type = "group", order = 20, name = 'Themes', childGroups = "tab", args = {
		Settings = { type = "group", order = 20, name = 'Theme Settings', childGroups = "tree", hidden = function() return not selectedSkin end, args = {} },
		Display  = { type = "group", order = 30, name = 'Display Conditions', childGroups = "tab",  hidden = function() return not selectedSkin end, args = {} },
	} },
	Profiles = { type = "group", order = 30, name = 'Profiles', childGroups = nil, args = {} },
} }	

--================================================

local fontFlagsValues = { 
	["ZDEFAULT"] = "Default",
	[""]        = "Soft",
	["OUTLINE"] = "Soft Thin", 
	["THICKOUTLINE"] = "Soft Thick", 
	["MONOCHROME"] = "Sharp",
	["MONOCHROME, OUTLINE"] = "Sharp Thin", 
	["MONOCHROME, THICKOUTLINE"] = "Sharp Thick", 
}

--================================================

local function SetCVarCombat(key, value)
	if InCombatLockdown() then
		print("You are in combat, settings cannot be updated.")
	else
		if type(value) == 'boolean' then 
			SetCVar(key, value and "1" or "0")
		else	
			SetCVar(key, value)
		end
	end
end

--================================================

local function Opt_GetOption( tree, order )
	local opt = addon.OptionsTable
	for _,key in ipairs( {strsplit('/', tree)} ) do
		if not opt.args[key] then 
			opt.args[key] = { type = "group", order = order, name = key, args = {} } 
		end
		opt = opt.args[key]
	end
	return opt, opt.args
end

local Opt_SetupOption
do
	local index = 1
	Opt_SetupOption = function( tree, name, options, order )
		local opt, args = Opt_GetOption(tree, index+100)
		if (not name) or args[name] then
			args = name and args[name].args or args
			for k,v in pairs(options) do args[k] = v end
		else
			args[name] = { type = 'group', name = name, order = order or index, inline = not opt.childGroups, args = options }
		end	
		index = index + 1		
		return options
	end
end

local Opt_GetSkinsValues
do		
	local list = {}
	function Opt_GetSkinsValues()
		wipe(list)
		for index,skin in next, addon.db.skins do 
			list[index] = skin.__skinName or 'Default'
		end
		return list
	end
end

local function Opt_SwapSkins(i,j)
	local skins = addon.db.skins
	skins[i], skins[j] = skins[j], skins[i]
	local rules = addon.db.rules
	rules[i], rules[j] = rules[j], rules[i]
end

local Opt_MakeSelectedSkinConditions
do
	local convert = { ['true'] = true, ['false'] = false }
	local keys = { type = 'type', classification = 'classification', reaction = 'reaction', level = 'level', names = 'names', target = 'target' }
	local def_values = { type = 'Player' , reaction = 'hostile', level = 100, names = "", classification = 'elite', target = true }
	local values = {
		type = { Player = 'Player', Creature = 'Creature', Pet = 'Pet', GameObject = 'Game object', Vehicle = 'Vehicle', Vignette = 'Vignette', 
				 ['~=Player'] = 'not Player', ['~=Creature'] = 'not Creature', ['~=Pet'] = 'not Pet', ['~=GameObject'] = 'not Game Object', ['~=Vehicle'] = 'not Vehicle', ['~=Vignette'] = 'not Vignette',
				 ['~~'] = '|cFFff0000delete|r' },
		classification = { boss = 'Boss', elite = 'Elite', rare = 'Rare', rareelite = 'Rare Elite', normal = 'Normal', trivial = "Trivial",  minus = "Minus", 
				           ['~=boss'] = 'not Boss', ['~=elite'] = 'not Elite', ['~=rare'] = 'not Rare', ['~=rareelite'] = 'not Rare Elite', ['~=normal'] = 'not Normal', ['~=trivial'] = "not Trivial",  ['~=minus'] = "not Minus", 
					       ['~~'] = '|cFFff0000delete|r' },
		reaction =  { hostile = 'Hostile', neutral = 'Neutral', friendly = 'Friendly', 
					 ['~=hostile'] = 'not Hostile', ['~=neutral'] = 'not Neutral', ['~=friendly'] = 'not Friendly', 
					 ['~~'] = '|cFFff0000delete|r' },
		level    = { ['-1'] = '??', ['100'] = '100', ['101'] = '101', ['102'] = '102', ['~~'] = '|cFFff0000delete|r' },
		target   = { ['true'] = 'true', ['false'] = 'false', ['~~'] = '|cFFff0000delete|r' },
	}
	
	function MakeConditions(skinRules, options)
		local function MakeCondition(key, index)
			local cond = {
				name = key,
				get = function(info) 
					local comp  = skinRules[index][2]
					local value = skinRules[index][3]
					return ( comp == '==' ) and tostring(value) or  (comp .. tostring(value))
				end,
				set = function(info, value)	
					if value == '~~' or (key == 'names' and strtrim(value)=='') then
						table.remove( skinRules, index )
						MakeConditions( skinRules, options )
					else
						local comp, val = '==', tonumber(value) or convert[value]
						if val==nil then
							local ncomp = strsub(value,1,2) 
							if ncomp == '<=' or ncomp == '>=' or ncomp == '~=' then
								val  = tonumber(strsub(value,3)) or strsub(value,3)
								comp = ncomp
							end
						end
						skinRules[index][2] = comp
						if val~=nil then
							skinRules[index][3] = val
						else
							skinRules[index][3] = value
						end
					end	
					addon:Update(false)					
				end,
				confirm = function(info, value)
					if value == '~~' then 
						return 'Are you sure you want to remove this condition ?' 
					elseif key=='names' and strtrim(value)=='' then
						return 'Names field is empty. Do you want to remove this condition ?'
					end
				end,
			}	
			if key == 'names' then
				cond.type = 'input'
				cond.multiline = 10
				cond.width = 'full'
				cond.values = nil
				cond.order = 99
				options[key] = cond
			else
				cond.type = 'select'
				-- cond.width = 'half'
				cond.order = index
				cond.values = values[key] or {}
				options[key..index] = cond
			end
		end
		wipe(options)
		for i=2,#skinRules do
			MakeCondition( skinRules[i][1], i )
		end
		-- options.headercond1 = { type = 'header', order = 1.1, name = 'Activation Rules' }
		options.headercond2 = { type = 'header', order = 1.9, name = 'Conditions' }
		options.AndOr = {
			type = 'select', width = 'normal', order = 1.5,
			name = 'Required for activation',
			desc = '',
			get = function() return skinRules[1] end,
			set = function(info,key)
				skinRules[1] = key
				addon:Update(false)
			end,
			values = { ['and'] = 'All conditions', ['or'] = 'Any condition' },
		}
		options.toggle = {
			type = 'select', width = 'normal', order = 1.6,
			name = 'Add a New Condition',
			desc = 'Select a condition variable to add.',
			get = function() end,
			set = function(info,key)
				if key~='names' or not options[key] then
					skinRules[#skinRules+1] = { key, '==', def_values[key] }	
					MakeCondition( key, #skinRules )
					addon:Update(false)					
				end	
			end,
			values = keys,
		}
	end
	Opt_MakeSelectedSkinConditions = function()
		MakeConditions( addon.db.rules[selectedSkinIndex], Opt_GetOption( 'Skins/Display' ).args )
	end
end

local function Opt_SetSelectedSkin(key)
	selectedSkinIndex = key
	selectedSkin = addon.db.skins[key]
	Opt_MakeSelectedSkinConditions()
end

--------------------------------------------------------
-- General Options
--------------------------------------------------------

Opt_SetupOption( 'General', 'Visibility', {
	nameplateShowAll = {
		type = "toggle",
		order = 11, width = "double",
		name = "Show all Nameplates",
		desc = "Show nameplates for all nearby units. If disabled only show relevant units when you are in combat.",
		get = function() return GetCVar ("nameplateShowAll") == "1" end,
		set = function (_, value) SetCVarCombat("nameplateShowAll", value) end,
	},
	separator1 = { type = 'description', name = "", order = 50, },	
	nameplateShowEnemies = {
		type = "toggle",
		order = 101,
		name = "Show Enemies",
		desc = "Show nameplates for enemies.",
		get = function() return GetCVar ("nameplateShowEnemies") == "1" end,
		set = function (_, value) SetCVarCombat("nameplateShowEnemies", value) end,
	},
	nameplateShowEnemyMinions = {
		type = "toggle",
		order = 102, width = 'half',
		name = "Minions",
		desc = "Show nameplate for enemy pets, totems and guardians.",
		get = function() return GetCVar ("nameplateShowEnemyMinions") == "1" end,
		set = function (_, value) SetCVarCombat("nameplateShowEnemyMinions", value) end,
		disabled = function() return GetCVar ("nameplateShowEnemies") == "0" end,			
	},
	nameplateShowEnemyMinus = {
		type = "toggle",
		order = 103, width = 'half',
		name = "Minor",
		desc = "Show nameplate for minor enemies.",
		get = function() return GetCVar ("nameplateShowEnemyMinus") == "1" end,
		set = function (_, value) SetCVarCombat ("nameplateShowEnemyMinus", value) end,
		disabled = function() return GetCVar ("nameplateShowEnemies") == "0" end,
	},
	separator2 = { type = 'description', name = "", order = 105, },
	nameplateShowFriends = {
		type = "toggle",
		order = 201,
		name = "Show Friends",
		desc = "Show nameplates for friendly units.",
		get = function() return GetCVar ("nameplateShowFriends") == "1" end,
		set = function (_, value) SetCVarCombat("nameplateShowFriends", value) end,
	},
	nameplateShowFriendlyGuardians = {
		type = "toggle",
		order = 202, width = 'half',
		name = "Guardians",
		desc = "Show nameplates for friendly guardians.",
		get = function() return GetCVar ("nameplateShowFriendlyGuardians") == "1" end,
		set = function (_, value) SetCVarCombat("nameplateShowFriendlyGuardians", value) end,
		disabled = function() return GetCVar ("nameplateShowFriends") == "0" end,			
	},
	nameplateShowFriendlyTotems = {
		type = "toggle",
		order = 203, width = 'half',
		name = "Totems",
		desc = "Show nameplates for friendly totems.",
		get = function() return GetCVar("nameplateShowFriendlyTotems") == "1" end,
		set = function (_, value) SetCVarCombat("nameplateShowFriendlyTotems", value) end,
		disabled = function() return GetCVar ("nameplateShowFriends") == "0" end,
	},
	nameplateShowFriendlyMinions = {
		type = "toggle", 
		order = 204, width = 'half',
		name = "Minions",
		desc = "Show nameplates for friendly minions.",
		get = function() return GetCVar("nameplateShowFriendlyMinions") == "1" end,
		set = function (_, value) SetCVarCombat("nameplateShowFriendlyMinions", value) end,
		disabled = function() return GetCVar ("nameplateShowFriends") == "0" end,
	},
	nameplateShowFriendlyPets = {
		type = "toggle",
		order = 205, width = 'half',
		name = "Pets",
		desc = "Show nameplates for friendly pets.",
		get = function() return GetCVar("nameplateShowFriendlyPets") == "1" end,
		set = function (_, value) SetCVarCombat("nameplateShowFriendlyPets", value) end,
		disabled = function() return GetCVar ("nameplateShowFriends") == "0" end,			
	},
} )	
	
Opt_SetupOption( 'General', 'Stacking', {
	nameplateMotion = {
		type = "toggle",
		order = 301,
		name = "Stack Nameplates",
		desc = "Nameplates must not overlap each other.",
		get = function() return GetCVar ("nameplateMotion") == "1" end,
		set = function (_, value) SetCVarCombat("nameplateMotion", value) end,
	},
	nameplateMotionSpeed = {
		type = "range", min = 0.001, max = 0.2, step = 0.005, 
		order = 302,			
		name = "Motion Speed",
		desc = "How fast the nameplate moves when stacking is enabled (default: 0.025)",
		get = function() return tonumber (GetCVar ("nameplateMotionSpeed")) end,
		set = function (_, value) SetCVar ("nameplateMotionSpeed", value) end,
		disabled = function() return GetCVar ("nameplateMotion") == "0" end,
	},
	nameplateOverlapV = {
		type = "range",	min = 0.2,	max = 1.6,	step = 0.1,
		order = 303,			
		name = "Vertical Padding",
		desc = "Verticaly distance between each nameplate (default: 1.10)",
		get = function() return tonumber (GetCVar("nameplateOverlapV")) end,
		set = function (_, value) SetCVarCombat("nameplateOverlapV", value)	end,
		disabled = function() return GetCVar ("nameplateMotion") == "0" end,			
	},		
} )	

Opt_SetupOption( 'General', 'Miscellaneous', {
	nameplateGlobalScale = {
		type = "range", min = 0.5, 	max = 2, step = 0.1,
		order = 401,
		name = "Global Scale",
		desc = "Scale for all nameplates.",
		get = function() return tonumber(GetCVar("nameplateGlobalScale")) end,
		set = function (_, value) SetCVarCombat("nameplateGlobalScale", value) end,
	},
	nameplateSelectedScale = {
		type = "range", min = 0.5, 	max = 2, step = 0.1,
		order = 402,			
		name = "Target Scale",
		desc = "The nameplate Scale for the target.",
		get = function() return tonumber(GetCVar("nameplateSelectedScale")) end,
		set = function(_, value) SetCVarCombat("nameplateSelectedScale", value) end,
	},
	nameplateMinScale = {
		type = "range", min = 0.5, 	max = 1, step = 0.1,
		order = 403,			
		name = "Far Away Scale",
		desc = "Scale when the nameplate is far away from the camera.",
		get = function() return tonumber(GetCVar("nameplateMinScale")) end,
		set = function (_, value) SetCVarCombat("nameplateMinScale", value) end,
	},
	nameplateMinAlpha = {
		type = "range", min = 0, max = 1, step = 0.1,
		order = 404,
		name = "Far away Transparency",
		desc = "Alpha when the nameplate is far away from the camera (default: 0.5)",
		get = function() return tonumber (GetCVar ("nameplateMinAlpha")) end,
		set = function (_, value) SetCVarCombat("nameplateMinAlpha", value)	end,
	},
	nameplateMaxDistance = {
		type = "range", min = 1, max = 100,	step = 1,
		order = 405,
		name = "Max. Distance",
		desc = "How far you can see nameplates (in yards). |cFFFFFFFFDefault: 60|r",
		get = function() return tonumber(GetCVar("nameplateMaxDistance")) end,
		set = function (_, value) SetCVarCombat("nameplateMaxDistance", value) end,
	},
	nameplateOtherAtBase = {
		type = "select",
		order = 406,			
		name = "Nameplates Anchor",
		desc = "Where the nameplate is anchored to.",
		get = function() return GetCVar("nameplateOtherAtBase") end,
		set = function (_, value) SetCVarCombat("nameplateOtherAtBase", value) end,
		values = {	['0'] = "Top", ['1'] = "Top&Bottom", ['2'] = "Bottom"  },
	},
	ShowClassColorInNameplate = {
		type = "toggle",
		order = 407,
		name = "Show Class Colors",
		desc = "Display class colors for enemy players.",
		get = function() return GetCVar ("ShowClassColorInNameplate") == "1" end,
		set = function (_, value) SetCVarCombat("ShowClassColorInNameplate", value) end,
	},
	ShowNamePlateLoseAggroFlash = {
		type = "toggle",
		order = 408,
		name = "Flash when losing Aggro",
		desc = "Flash when losing Aggro.",
		get = function() return GetCVar ("ShowNamePlateLoseAggroFlash") == "1" end,
		set = function (_, value) SetCVarCombat("ShowNamePlateLoseAggroFlash", value) end,
	},
	NamesAlwaysVisible = {
		type = "toggle",
		order = 409,
		name = "Names Always Visible",
		desc = "Always display unit names.",
		get = function() return addon.db.general.namesAlwaysVisible end,
		set = function (_, value) 
			addon.db.general.namesAlwaysVisible = value or nil 
			addon:Update(true)
		end,
	},
} )	

Opt_SetupOption( 'General', 'Personal Resources', {
	nameplateShowSelf = {
		type = "toggle",
		order = 901,
		name = "My Health&Power Bars",
		desc = "Shows health and power bars under your character.",
		get = function() return GetCVar("nameplateShowSelf") == "1" end,
		set = function ( _, value) SetCVarCombat("nameplateShowSelf", value)	end,
	},
	nameplateResourceOnTarget = {
		type = "toggle",
		order = 902,		
		name = "My resources on Target",
		desc = "Shows resources such as combo points above your target. Personal Health and Power Bars must be enabled",
		get = function() return GetCVar ("nameplateResourceOnTarget") == "1" end,
		set = function (_, value) SetCVarCombat("nameplateResourceOnTarget", value) end,
		disabled = function() return GetCVar ("nameplateShowSelf") == "0" end,
	},
} )	

--------------------------------------------------------
-- Skins Management Options
--------------------------------------------------------

Opt_SetupOption( 'Skins', nil, {
	selectedSkin = {
		type = 'select',
		order = 1,
		name = 'Theme',
		get = function() 
			if not selectedSkin then Opt_SetSelectedSkin(1)	end
			return selectedSkinIndex  
		end,
		set = function(info, key) Opt_SetSelectedSkin(key) end,
		values = Opt_GetSkinsValues,
	},
	moveUp = {
		type = 'execute',
		order = 2,
		width = 'half',
		name = 'Up',
		desc = 'Increase theme display priority',
		func = function() 
			Opt_SwapSkins(selectedSkinIndex, selectedSkinIndex-1) 
			Opt_SetSelectedSkin(selectedSkinIndex-1)
		end,
		hidden = function() return not (selectedSkin and selectedSkinIndex>2) or skinCommand~=nil end,
	},
	moveDown = {
		type = 'execute',
		order = 3,
		width = 'half',
		name = 'Down',
		desc = 'Decrease theme display priority',
		func = function() 
			Opt_SwapSkins(selectedSkinIndex, selectedSkinIndex+1) 
			Opt_SetSelectedSkin(selectedSkinIndex+1)
		end,
		hidden = function() return not (selectedSkin and selectedSkinIndex>1 and  selectedSkinIndex<#addon.db.skins) or skinCommand~=nil  end,
	},
	deleteSkin = {
		type = 'execute',
		order = 6,
		width = 'half',
		name = 'Delete',
		desc = 'Delete selected Theme',
		func = function() 
			tremove( addon.db.skins, selectedSkinIndex )
			tremove( addon.db.rules, selectedSkinIndex )
			Opt_SetSelectedSkin( 1 )
			addon:Update(true)
		end,
		confirm = function() return 'Are you sure you want to delete the selected theme ?' end,
		hidden = function() return skinCommand~=nil or (not selectedSkin) or selectedSkinIndex==1 end,
	},
	resetSkin = {
		type = 'execute',
		order = 4,
		width = 'half',
		name = 'Reset',
		desc = 'Assign Blizzard default values to the selected theme.',
		func = function()  
			local name = selectedSkin.__skinName
			wipe(selectedSkin)
			selectedSkin.__skinName = name
			addon:Update(true)	
		end,
		confirm = function() return 'Are you sure you want to assign blizzard default values to the selected theme ?' end,
		hidden = function() return skinCommand~=nil or (not next(addon.db.skins[1])) end,
	},
	createSkin = {
		type = 'execute',
		order = 7,
		width = 'half',
		name = 'Add',
		desc = 'Create a New Theme',
		func = function() newSkinName = ''; skinCommand = 'create' end,
		hidden = function() return skinCommand~=nil end,
	},
	newSkinName = {
		type = 'input',
		name = 'Type the Theme Name',
		order = 10,
		get = function() return newSkinName end,
		set = function(info,skinName) 
			local index = #addon.db.skins+1
			addon.db.skins[index] = CopyTable(selectedSkin)
			addon.db.skins[index].__skinName = skinName 
			addon.db.rules[index] = { 'and' }
			Opt_SetSelectedSkin(index)
			skinCommand = nil
			newSkinName = ''
		end,
		hidden = function() return not skinCommand end,
		validate = function(info,value) return not addon.db.skins[value] end,
	},
	newSkinCancel = {
		type = 'execute',
		order = 20,
		width = 'half',
		name = 'Cancel',
		func = function() skinCommand = nil; newSkinName = '' end,
		hidden = function() return not skinCommand end,
	},
} )

--------------------------------------------------------
-- Selected Skin Options
--------------------------------------------------------

Opt_SetupOption( 'Skins/Settings', 'Health Bar', {
	barDisabled = {
		type = "toggle",
		order = 1, width = "double",
		name = "Hide Health Bar",
		get = function() return selectedSkin.healthBarHidden end,
		set = function (_, value) 
			selectedSkin.healthBarHidden = value or nil
			addon:Update(true)
		end,
	},
	header1 = { type = "header", order = 5, name = "Health Bar" },
	barWidth = {
		type = 'range', order = 11, name = 'Health Bar Width', min = 0, softMax = 200, step = 1,
		get = function() return selectedSkin.healthBarWidth	or 110 end,
		set = function(info,value)
			selectedSkin.healthBarWidth = value~=110 and value or nil
			addon:Update(true)					
		end,
		disabled = function() return selectedSkin.healthBarHidden end,		
	},
	barHeight =  {
		type = 'range', order = 12, name = 'Health Bar Height', min = 1, softMax = 64, step = 1,
		get = function() return selectedSkin.healthBarHeight or 4 end,
		set = function(info,value)
			selectedSkin.healthBarHeight = value~=4 and value or nil
			addon:Update(true)
		end,
		disabled = function() return selectedSkin.healthBarHidden end,				
	},
	barTexture = {
		type = "select", dialogControl = "LSM30_Statusbar",
		order = 13,
		name = "Bar Texture",
		desc = "Adjust the bar texture.",
		get = function (info) return selectedSkin.healthBarTexture or "Blizzard NamePlate" end,
		set = function (info, v)
			selectedSkin.healthBarTexture = v
			addon:Update(true)					
		end,
		values = AceGUIWidgetLSMlists.statusbar,
		disabled = function() return selectedSkin.healthBarHidden end,		
	},
} )	
		
Opt_SetupOption( 'Skins/Settings', 'Name Text', {
	nameFontFile = {
		type = "select", dialogControl = "LSM30_Font",
		order = 21,
		name = "Font Name",
		values = AceGUIWidgetLSMlists.font,
		get = function () return selectedSkin.nameFontFile or 'Friz Quadrata TT' end,
		set = function (_, v)
			selectedSkin.nameFontFile = v
			addon:Update(true)
		end,				
	},
	nameFontFlags = {
		type = "select",
		order = 22,
		name = "Font Border",
		values = fontFlagsValues,
		get = function () return selectedSkin.nameFontFlags or "ZDEFAULT" end,
		set = function (_, v)
			selectedSkin.nameFontFlags =  v ~= "ZDEFAULT" and v or nil
			addon:Update(true)
		end,
		
	},
	nameFontSize = {
		type = "range",
		order = 23,
		name = 'Font Size',
		min = 0,
		softMax = 23,
		step = 1,
		get = function () return selectedSkin.nameFontSize or 11 end,
		set = function (_, v)
			selectedSkin.nameFontSize = v>0 and v or nil
			addon:Update(true)
		end,				
	},
	nameOffsetX =  {
		type = 'range', order = 10, name = 'X Adjust', min = -32, softMax = 32, step = 1,
		get = function() return selectedSkin.nameOffsetX or 0 end,
		set = function(info,value)
			selectedSkin.nameOffsetX = value~=0 and value or nil
			addon:Update(true)
		end,
	},
	nameOffsetY =  {
		type = 'range', order = 11, name = 'Y Adjust', min = -32, softMax = 32, step = 1,
		get = function() return selectedSkin.nameOffsetY or 4 end,
		set = function(info,value)
			selectedSkin.nameOffsetY = value~=4 and value or nil
			addon:Update(true)
		end,
	},
} )

Opt_SetupOption( 'Skins/Settings', 'Cast Bar', {
	barDisabled = {
		type = "toggle",
		order = 1, width = "double",
		name = "Hide Cast Bar",
		get = function() return selectedSkin.castBarHidden end,
		set = function (_, value) 
			selectedSkin.castBarHidden = value or nil
			addon:Update()
		end,
	},
	header1 = { type = "header", order = 5, name = "Cast Bar" },
	barHeight =  {
		type = 'range', order = 10, name = 'Cast Bar Height', min = 1, softMax = 64, step = 1,
		get = function() return selectedSkin.castBarHeight or 8 end,
		set = function(info,value)
			selectedSkin.castBarHeight = value~=8 and value or nil
			addon:Update(true)
		end,
		disabled = function() return selectedSkin.castBarHidden end,
	},
	barTexture = {
		type = "select", dialogControl = "LSM30_Statusbar",
		order = 11,
		name = "Bar Texture",
		desc = "Adjust the bar texture.",
		get = function (info) return selectedSkin.castBarTexture or "Blizzard NamePlate" end,
		set = function (info, v)
			selectedSkin.castBarTexture = v
			addon:Update(true)					
		end,
		values = AceGUIWidgetLSMlists.statusbar,
		disabled = function() return selectedSkin.castBarHidden end,		
	},
	header2 = { type = "header", order = 20, name = "Spell Text" },
	cbFontFile = {
		type = "select", dialogControl = "LSM30_Font",
		order = 21,
		name = "Font Name",
		values = AceGUIWidgetLSMlists.font,
		get = function () return selectedSkin.castBarFontFile or 'Friz Quadrata TT' end,
		set = function (_, v)
			selectedSkin.castBarFontFile = v
			addon:Update(true)
		end,
		disabled = function() return selectedSkin.castBarHidden end,		
	},
	cbFontFlags = {
		type = "select",
		order = 22,
		name = "Font Border",
		values = fontFlagsValues,
		get = function () return selectedSkin.castBarFontFlags or "ZDEFAULT" end,
		set = function (_, v)
			selectedSkin.castBarFontFlags =  v ~= "ZDEFAULT" and v or nil
			addon:Update(true)
		end,
		disabled = function() return selectedSkin.castBarHidden end,
	},
	cbFontSize = {
		type = "range",
		order = 23,
		name = 'Font Size',
		min = 0,
		softMax = 23,
		step = 1,
		get = function () return selectedSkin.castBarFontSize or 8 end,
		set = function (_, v)
			selectedSkin.castBarFontSize = v>0 and v or nil
			addon:Update(true)
		end,				
		disabled = function() return selectedSkin.castBarHidden end,		
	},	
} )	

Opt_SetupOption( 'Skins/Settings', 'Raid Icon', {
	raidTargetSize =  {
		type = 'range', order = 33, name = 'Icon Size', min = 0, softMax = 64, step = 1,
		name = "Icon Size",
		desc = "Raid Target Icon Size",
		get = function() return selectedSkin.raidTargetSize or 22 end,
		set = function(info,value)
			selectedSkin.raidTargetSize = value~=22 and value or nil
			addon:Update(true)
		end,
	},
	raidTargetOffsetX =  {
		type = 'range', order = 31, name = 'X Adjust', softMin = -32, softMax = 120, step = 1,
		get = function() return selectedSkin.raidTargetOffsetX or -15 end,
		set = function(info,value)
			selectedSkin.raidTargetOffsetX = value~=-15 and value or nil
			addon:Update(true)
		end,
	},
	raidTargetOffsetY =  {
		type = 'range', order = 32, name = 'Y Adjust', softMin = -32, softMax = 32, step = 1,
		get = function() return selectedSkin.raidTargetOffsetY or 0 end,
		set = function(info,value)
			selectedSkin.raidTargetOffsetY = value~=0 and value or nil
			addon:Update(true)
		end,
	},
} )
	
Opt_SetupOption( 'Skins/Settings', 'Elite Icon', {	
	classIconSize =  {
		type = 'range', order = 43, name = 'Icon Size', min = 0, softMax = 32, step = 1,
		name = "Icon Size",
		desc = "Classification Icon Size. Set Zero to hide the Icon.",
		get = function() return selectedSkin.classIconSize or 14 end,
		set = function(info,value)
			selectedSkin.classIconSize = value~=14 and value or nil
			addon:Update(true)
		end,
	},
	classIconOffsetX =  {
		type = 'range', order = 41, name = 'X Adjust', softMin = -32, softMax = 120, step = 1,
		get = function() return selectedSkin.classIconOffsetX or 0 end,
		set = function(info,value)
			selectedSkin.classIconOffsetX = value~=0 and value or nil
			addon:Update(true)
		end,
		disabled = function() return selectedSkin.classIconSize == 0 end,				
	},
	classIconOffsetY =  {
		type = 'range', order = 42, name = 'Y Adjust', softMin = -32, softMax = 32, step = 1,
		get = function() return selectedSkin.classIconOffsetY or 0 end,
		set = function(info,value)
			selectedSkin.classIconOffsetY = value~=0 and value or nil
			addon:Update(true)
		end,
		disabled = function() return selectedSkin.classIconSize == 0 end,				
	},
} )

--=============================================================

function addon:GetSkin()
	return selectedSkin, selectedSkinIndex
end

function addon:SetupOptions( tree, name, options, order )
	return Opt_SetupOption( tree, name, options, order )
end

--=============================================================
